use strict;
$|++;

use Config;
open CONFIG_WEB, ">config.web" or die "Cannot open config.web for writing: $!";
print CONFIG_WEB "#!$Config{perlpath} -w\n";
while (<DATA>) { print  CONFIG_WEB $_; }
close CONFIG_WEB or die "Cannot close config.web: $!";
chmod 0755, 'config.web';
__DATA__
use strict;
$|++;

use OandA::Install;
use Getopt::Std;
use Cwd;

use vars qw/
	%config
	/;

my %pages = ( SearchMenu => 'search menu:index',
	NameHintsPage => 'name search hints:hints_name',
	DateHintsPage => 'date search hints:hints_date',
	DescHintsPage => 'armory description search hints:hints_desc',
	NpHintsPage => 'name pattern search hints:hints_np',
	BpHintsPage => 'blazon pattern search hints:hints_bp',
	ComplexHintsPage => 'complex search hints:hints_complex',
	OverviewPage => 'about SCA heraldry:heraldry_overview',
	LimitPage => 'about search limits:search_limits',
	DownloadPage => 'about obtaining the database:data_obtain',
	DbFormatPage => 'about the database format:data_format',
	DbSymbolsPage => 'non-ASCII symbols:data_symbols',
);

my %scripts = (
	Glossary => 'glossary:glossary',
	Copyright => 'database copyright:data_copyright',
	Version => 'database version:version',
	NameSearch => 'name search:oanda_name',
	DescSearch => 'armory description search:oanda_desc',
	NpSearch => 'name pattern search:oanda_np',
	BpSearch => 'blazon pattern search:oanda_bp',
	DateSearch => 'date/kingdom search:oanda_date',
	ComplexSearch => 'complex search:oanda_complex',
  #  'Correction', 'correction request:correction'
);

my %template = map {$_, "templates/".lc($_)} keys %pages, keys %scripts, 
	map "IndexPage$_", ("", "A".."Z");


my $conf_file = '.configweb';
my $config_version = 20010508;

my $usage = <<USAGE;
config.web [-c] [-f] [-r]
   -c  cgibin
   -f  force defaults
   -r  use RCS
USAGE

die $usage unless getopts("cfr");

Force_Defaults(defined $Getopt::Std::opt_f); 
Use_RCS(defined $Getopt::Std::opt_r);
CGI_bin(defined $Getopt::Std::opt_c);
# bogus references to shut up -w warning about single use...
if (defined($Getopt::Std::opt_f) || defined($Getopt::Std::opt_r) || defined($Getopt::Std::opt_c)) {}

#==========================================================#
# Print introductory messages to the config.web installer. #
#==========================================================#

print <<'EOF';

This Perl script configures and installs web-based tools for viewing
the SCA Ordinary and searching the SCA Armorial.

This portion of the installation should be performed ON THE WEBSERVER.
EOF

# then it slurps in the various files
$template{$_} = slurpfile($template{$_}) foreach keys %template;
my $CommonClientCode = slurpfile('OandA/Client.pm');
my $ConfigDbScript = slurpfile('templates/configdbscript');
my $Helpers = slurpfile('OandA/Install.pm');
$config{XXHelpersXX} = $Helpers;

#=====================================================#
# config.web has now slurped up all the files to be   #
# installed on the webserver.                         #
#=====================================================#

print "\nThe following web pages will be installed on the webserver:\n";
print_list(sort values %pages);
print "\nThe following CGI scripts will be installed on the webserver:\n";
print_list(sort values %scripts);
print "\nThe following other files will also be installed on the webserver:\n";
print_list(
	'ordinary master index',
	'common Perl code',
	'database server config script',
	'ordinary index pages(A-Z)',
	);

print "\nPress Enter to proceed, or else interrupt the installation now.\n";
input('');

# get old config values if available
read_config_file($conf_file);

# now set some 'constants'
$config{XXPrimerUrlXX} = 'http://www.sca.org/heraldry/primer';
$config{XXLoARUrl} = 'http://www.sca.org/heraldry/loar';
$config{XXVersionXX} = $config_version;
$config{XXHeadXX} = '';
$config{XXTrailerXX} = "<HR><Address>config.web version $config_version</ADDRESS>";
$config{XXTrailer2XX} = '';
$config{XXCloseHtmlXX} = "</BODY></HTML>";

#============================#
# Set webserver parameters.  #
#============================#

machine_questions();

# get ServerName, ServerPort

my $ServerName = $config{XXServerNameXX} || 'www.sca.org';
while (1)
{
	print <<EOF;

What is the DNS hostname of this machine?

  (This should be listed as the ServerName
  in the webserver configuration file.)

EOF
	print "[$ServerName] ";
	$ServerName = input($ServerName);
	last if gethostbyname($ServerName);
	print "Sorry, '$ServerName' is not an accessible host.\n";
	print "Try again...\n";
}
$config{XXServerNameXX} = $ServerName;

my $ServerPort = $config{XXServerPortXX} || 80;
while (1)
{
	print <<'EOF';

To which port does this machine listen for incoming HTTP requests?

  (This should be listed as the ServerPort in the webserver
  configuration file.)

EOF
	print "[$ServerPort] ";
	$ServerPort = input($ServerPort);
	last if $ServerPort >= 1 && $ServerPort <= 32767;
	print "Sorry, '$ServerPort' is not a valid port.\n";
	print "Try again...\n";
}
$config{XXServerPortXX} = $ServerPort;

# get DocumentRoot

my $DocumentRoot = $config{XXDocumentRootXX} || '/local';
$DocumentRoot = join('/', cwd, $DocumentRoot) unless $DocumentRoot =~ m:^/:;
while (1)
{
	print <<EOF;

What is the pathname of the directory corresponding to
'http://$ServerName:$ServerPort/'?

  (This should be listed as the DocumentRoot
  in the webserver configuration file.)
  

EOF
	print "[$DocumentRoot] ";
	$DocumentRoot = input($DocumentRoot);
	$DocumentRoot = join('/', cwd, $DocumentRoot) unless $DocumentRoot =~ m:^/:;
	last if -e $DocumentRoot && -d $DocumentRoot;
	print "Sorry, '$DocumentRoot' does not exist.\n" unless -e $DocumentRoot;
	print "Sorry, '$DocumentRoot' is not a directory.\n" unless -d $DocumentRoot;
	print "Try again...\n"
}
$config{XXDocumentRootXX} = $DocumentRoot;

# set ScriptAlias if CGI_bin

if (CGI_bin)
{
	my $ScriptAlias = $config{XXScriptAliasXX} || '/usr/local/etc/httpd/cgi-bin';
	$ScriptAlias = join('/', cwd, $ScriptAlias) unless $ScriptAlias =~ m:^/:;
	while (1)
	{
		print <<EOF;
	
	What is the pathname of the directory corresponding to
	'http://$ServerName:$ServerPort/cgi-bin/'?
	
	  (This should be listed as the ScriptAlias
	  in the webserver configuration file.)
	  
	
EOF
		print "[$ScriptAlias] ";
		$ScriptAlias = input($ScriptAlias);
		$ScriptAlias = join('/', cwd, $ScriptAlias) unless $ScriptAlias =~ m:^/:;
		last if -e $ScriptAlias && -d $ScriptAlias;
		print "Sorry, '$ScriptAlias' does not exist.\n" unless -e $ScriptAlias;
		print "Sorry, '$ScriptAlias' is not a directory.\n" unless -d $ScriptAlias;
		print "Try again...\n"
	}
	$config{XXScriptAliasXX} = $ScriptAlias;
}

#====================================#

print <<'EOF';


Now you get to choose where the files will be installed.

For each group of files:
 +  The default location is given in [brackets].
 +  To override the default location, enter your
      choice in the space next to the brackets.
 +  To accept the default location, enter a blank line.

EOF

#====================================#
# Set paths and URLs for HTML pages. #
#====================================#

get_html_url('XXPageUrlXX', 'O&A web pages', 
	get_dirpath('XXPagePathXX', 'O&A web pages', "$DocumentRoot/heraldry/OandA", $DocumentRoot));
get_html_url('XXWebDataUrlXX', 'database file', 
	get_filepath('XXWebDataPathXX', 'database file', join('/', $config{XXPagePathXX}, 'oanda.db'), $DocumentRoot));

set_page_info(keys %pages);

get_html_url('XXIndexDirUrlXX', 'SCA Ordinary index pages',
	get_dirpath('XXIndexDirPathXX', 'SCA Ordinary index pages', join('/', $config{XXPagePathXX}, 'ordinary'), $DocumentRoot));

#=====================================#
# Set paths and URLs for CGI scripts. #
#=====================================#

my $ScriptPath = get_dirpath('XXScriptPathXX', 'CGI scripts',
	CGI_bin() ? $config{XXScriptAliasXX} : $config{XXPagePathXX},
	CGI_bin() ? $config{XXScriptAliasXX} : $config{XXDocumentRootXX});
get_cgi_url('XXScriptUrlXX', 'CGI scripts', $ScriptPath);

set_script_info(keys %scripts);

#========================#
# Set $CommonClientPath. #
#========================#

{
	my $foo;
	if (CGI_bin) { $config{XXScriptAliasXX} =~ m:^(.*)/[^/]+/?$:; $foo = $1 }
	else { $config{XXPagePathXX} =~ m:^(.*)/[^/]+/?$:; $foo = $1 }
	get_filepath('XXCommonClientPathXX', 'common Perl code', join('/', $foo, 'OandA', 'Client.pm'), $foo);
	($config{XXCommonClientLibPathXX} = $config{XXCommonClientPathXX}) =~ s:/OandA/Client.pm$::;
}

#=========================#
# Set $CorrectionLogPath. #
#=========================#

# $CorrectionLogPath = &get_filepath ('XXCorrectionLogPathXX',
#   'correction log', 'correction.log', '/');

#====================================#
# Set parameters of database server. #
#====================================#

print <<'EOF';


Now I need you to answer a few questions about the database server.
This may or may not be the same machine as the webserver, but
it must be accessable via TCP/IP and it must have disk space to
install the Armorial database (currently over 6 Mb).

For each question:
 +  The default answer is given in [brackets].
 +  To override the default answer, enter the
      correct answer in the space next to the brackets.
 +  To accept the default answer, enter a blank line.

EOF

# set DataHost, DataPort

my $DataHost = $config{XXDataHostXX} || 'www.sca.org';
while (1)
{
	print <<EOF;

What is the DNS hostname of the database server?

EOF
	print "[$DataHost] ";
	$DataHost = input($DataHost);
	last if gethostbyname($DataHost);
	print "Sorry, '$DataHost' is not an accessible host.\n";
	print "Try again...\n";
}
$config{XXDataHostXX} = $DataHost;

my $DataPort = $config{XXDataPortXX} || 1066;
while (1)
{
	print <<'EOF';

To which port will the database server
listen for incoming database requests?


EOF
	print "[$DataPort] ";
	$DataPort = input($DataPort);
	last if $DataPort >= 1 && $DataPort <= 32767;
	print "Sorry, '$DataPort' is not a valid port.\n";
	print "Try again...\n";
}
$config{XXDataPortXX} = $DataPort;

get_filepath('XXConfigDbPathXX', 'database server config script', 'configdb', '/');

#=======================================#
# Save the new webserver configuration. #
#=======================================#

save_config($conf_file);

#============================================#
# Configure and install the webserver files. #
#============================================#

page_install($template{$_}, $_) foreach keys %pages;

install($template{IndexPage}, "$config{XXIndexDirPathXX}/index.html", 0444, 'ordinary master index');

for my $letter ('A' .. 'Z')
{
	install($template{"IndexPage$letter"}, "$config{XXIndexDirPathXX}/$letter.html", 0444, "ordinary index page ($letter)");
}

script_install($template{$_}, $_) foreach keys %scripts;

key_install($CommonClientCode, 'CommonClient', 0444, 'common Perl code');
$config{XXInstallPackageXX} = slurpfile('OandA/Install.pm');
key_install($ConfigDbScript, 'ConfigDb', 0555, 'database server config script');

#=================================#
# Final message to the installer. #
#=================================#

print "\nYou have successfully installed the webserver files.\n";
print "Now copy `$config{XXConfigDbPathXX}' to the machine `$config{XXDataHostXX}'\n";
print " and execute `perl $config{XXConfigDbPathXX}' there.\n";

exit 0;

#==============================================#
# Functions specific to the config.web script. #
#==============================================#

# config.web function to install an HTML page in a file,
# taking the path from %config.

sub page_install
{
	my ($text, $tag) = @_;
	my ($what, $file) = split(/:/, $pages{$tag});
	key_install($text, $tag, 0444, $what);
}

# config.web function to install a CGI script in a file,
# taking the path from %config.

sub script_install
{
	my ($text, $tag) = @_;
	my ($what, $file) = split(/:/, $scripts{$tag});
	key_install($text, $tag, 0555, $what);
}

sub print_list
{
	my $i = 0;
	foreach my $file (@_)
	{
		my ($what, $name) = split(/:/, $file);
		printf " + %-35s", $what;
		print $i++%2 ? "\n" : ' ';
	}
	print $i%2 ? "\n" : ' ';
}

sub set_page_info # add tag to config
{
	foreach my $tag (@_)
	{
		my ($what, $file) = split(/:/, $pages{$tag});
		$config{"XX${tag}PathXX"} = $config{XXPagePathXX} . "/$file.html";
		$config{"XX${tag}UrlXX"} = $config{XXPageUrlXX} . "/$file.html";
	}
}

sub set_script_info # add tags to config
{
	foreach my $tag (@_)
	{
		my ($what, $file) = split(/:/, $scripts{$tag});
		$config{"XX${tag}PathXX"} = $config{XXScriptPathXX} . "/$file.cgi";
		$config{"XX${tag}UrlXX"} = $config{XXScriptUrlXX} . "/$file.cgi";
	}
}

sub get_html_url
{
	my ($tag, $what, $path) = @_;
	
	my $url = $config{$tag} || '';
	if ($url eq '')
	{
		$url = $path;
		$url =~ s/^$config{XXDocumentRootXX}//;
		$url = "http://$config{XXServerNameXX}:$config{XXServerPortXX}$url";
	}
	print "\nURL for $what:\n[$url] ";
	$url = &input ($url);
	$config{$tag} = $url;
	return $url;
}

sub get_cgi_url
{

	my ($tag, $what, $path) = @_;
	
	my $url = $config{$tag} || '';
	if ($url eq '')
	{
		$url = $path;
		if (CGI_bin)
		{
			$url =~ s/^$config{XXScriptAliasXX}//;
	  		$url = "http://$config{XXServerNameXX}:$config{XXServerPortXX}/cgi-bin$url";
		}
		else
		{
	  		$url =~ s/^$config{XXDocumentRootXX}//;
	  		$url = "http://$config{XXServerNameXX}:$config{XXServerPortXX}$url";
		}
	}
	print "\nURL for $what:\n[$url] ";
	$url = &input ($url);
	$config{$tag} = $url;
	return $url;
}

sub slurpfile
{
	my $file = shift;
	local ($/);
	open(PF, $file) or die "Couldn't read file '$file': $!";
	undef $/;
	return <PF>;
}
__END__
